#!/usr/bin/env lua

--
-- Various tests for module `fun`.
-- Tests for laziness.
-- Test for `fun.chain`, `fun.map`, `fun.enumerate`, `fun.each`,
--   `fun.reduce`, etc.
--

M = {}

local fun = require('fun')

local a = { 11, 22, 33, 44,  }
local b = { "aa", "bb", "cc",  }
local c = fun.range(1, 3)

local function task1(count, funs)
  fun.chain(a, b, c)
    :cycle()
    :take(count)
    :map(funs.f1)
    :map(funs.f2)
    :map(funs.f3)
    :map(funs.f4)
    :enumerate()
    :each(funs.show)
end

local function task2(count, funs)
  local sum = fun.chain(a, b, c)
    :cycle()
    :take(count)
    :map(funs.f1)
    :map(funs.f2)
    :map(funs.f3)
    :map(funs.f4)
    :reduce(funs.acc_fn, 0)
  print('sum:', sum)
end

M.test01 = function(count, quiet)
  quiet = quiet == true or false
  local verbose = true
  local calc_fn = function(x, n)
    local val
    if type(x) == 'number' then
      val = x * n
    elseif type(x) == 'string' then
      val = x .. '|' .. x
    else
      val = 'unknown'
    end
    return val
  end
  local make_map_fun = function(msg, n, func)
    local inner_fn = function(x)
      if not quiet then
        print(msg, x)
      end
      val = calc_fn(x, n)
      return val
    end
    return inner_fn
  end
  local funs = {}
  funs.f1 = make_map_fun('(f1) x:', 5)
  funs.f2 = make_map_fun('(f2) x:', 10)
  funs.f3 = make_map_fun('(f3) x:', 100)
  funs.f4 = make_map_fun('(f4) x:', 1000)
  funs.show = function(index, value)
    if verbose then
      print(string.format('result -- index: %d  value: %s', index, value))
      print('----')
    end
  end
  funs.acc_fn = function(acc, val)
    if type(val) == 'number' then
      acc = acc + val
    elseif type(val) == 'string' then
      acc = acc + #val
    end
    return acc
  end
  task1(count, funs)
  print('---------------------------------------------------')
  task2(count, funs)
end

M.main = function()
  print('test 11')
  print('-------')
  quiet = false
  if arg[1] == '-h' or arg[1] == '--help' then
    print('usage: lua test21.lua [-q | --quiet] [n]')
    os.exit()
  elseif arg[1] == '-q' or arg[1] == '--quiet' then
    quiet = true
    table.remove(arg, 1)
  end
  local count = 1
  if #arg == 1 then
    count = tonumber(arg[1])
  end
  M.test01(count, quiet)
end

if os.getenv('LUACLI') == nil then
  return M
else
  M.main()
end
